
/*
* FLYFISH TECHNOLOGIES Ltd.
* http://www.flyfish-tech.com
*
* FF32 chip usage examples
*   Updated: Aug 3, 2015
*
*
* This code is in the Public Domain. Use it at your own risk.
*/




#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdbool.h>

/* libudev-dev is required. You can inistall it by executing: sudo apt-get install libudev-dev */
#include <libudev.h>


#include "FF3x-lib.inc"


int main(int argc, char **argv) {

	char arr[32];
	int i;

	// locate FF32, initialize communication
	if(usb_GetPath(1) == false) {
		printf("*** Error locating FF32 USB device. Check USB cable. Unable to continue, exiting...\n");
		return EXIT_FAILURE; }
	if(usb_OpenComm() == false) {
		printf("*** Error allocating FF32 USB device for exclusive use. Unable to continue, exiting...\n");
		return EXIT_FAILURE; }



	// Get silicon version
	if(usb_GetChipInfo() == true) {
		if((usb_Buffer[1] == 0x0F) && (usb_Buffer[2] == 0x0B)) {
			printf("Chip is FF32, silicon version %d.%d\n", usb_Buffer[3], usb_Buffer[4]);
		} else {
			printf("Chip is not FF32"); }
	} else {
		printf("*** Error (GetChipInfo)...\n"); }



	// Set chip address
	if(usb_SetAddress(5) == true) {
		if(usb_Buffer[0] == 0x0E) {
			printf("Chip address has been set\n");
		} else {
			printf("*** Error setting the Chip address, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (SetAddress)...\n"); }



	// Get chip address
	if(usb_GetAddress() == true) {
		printf("Chip address is %d\n", usb_Buffer[1]);
	} else {
		printf("*** Error (GetAddress)...\n"); }



	// Set board Vendor
	strcpy(arr, "AAA Ltd.");
	if(usb_SetVendor((unsigned char *)&arr[0], strlen(arr)) == true) {
		printf("Board vendor has been set\n");
	} else {
		printf("*** Error (SetVendor)...\n"); }



	// Get board Vendor
	if(usb_GetVendor() == true) {
		printf("Board vendor is %s\n", (char *)&usb_Buffer[1]);
	} else {
		printf("*** Error (GetVendor)...\n"); }



	// Reset board vendor
	for(i=0; i<32; i++) {
		arr[i]= 0xFF; }
	if(usb_SetVendor((unsigned char *)&arr[0], 32) == true) {
		printf("Board vendor has been reset\n");
	} else {
		printf("*** Error (SetVendor)...\n"); }



	// Set board name
	strcpy(arr, "DemoBoard");
	if(usb_SetProduct((unsigned char *)&arr[0], strlen(arr)) == true) {
		printf("Board name has been set\n");
	} else {
		printf("*** Error (SetProduct)...\n"); }



	// Get board name
	if(usb_GetProduct() == true) {
		printf("Board name is %s\n", (unsigned char *)&usb_Buffer[1]);
	} else {
		printf("*** Error (GetProduct)...\n"); }



	// Reset board name
	for(i=0; i<32; i++) {
		arr[i]= 0xFF; }
	if(usb_SetProduct((unsigned char *)&arr[0], 32) == true) {
		printf("Board name has been reset\n");
	} else {
		printf("*** Error (SetProduct)...\n"); }



	// Set board serial
	strcpy(arr, "2014XZ73U");
	if(usb_SetSerialNumber((unsigned char *)&arr[0], strlen(arr)) == true) {
		printf("Board serial number has been set\n");
	} else {
		printf("*** Error (SetSerialNumber)...\n"); }



	// Get board serial
	if(usb_GetSerialNumber() == true) {
		printf("Board serial number is %s\n", (unsigned char *)&usb_Buffer[1]);
	} else {
		printf("*** Error (GetSerialNumber)...\n"); }



	// Reset board serial
	for(i=0; i<32; i++) {
		arr[i]= 0xFF; }
	if(usb_SetSerialNumber((unsigned char *)&arr[0], 32) == true) {
		printf("Board serial number has been reset\n");
	} else {
		printf("*** Error (SetSerialNumber)...\n"); }




	// Set digital output
	if(usb_SetDigitalOutput('B', 8, 1) == true) {
		if(usb_Buffer[0] == 0x0E) {
			printf("Digital output has been set\n");
		} else {
			printf("*** Error setting digital output, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (SetDigitalOutput)...\n"); }



	// Set multiple digital outputs
	arr[0]= 0x00;
	arr[1]= 0x3F;
	arr[2]= 0x00;
	arr[3]= 0x30;
	if(usb_SetBlockDigitalOutputs('A', (unsigned char *)&arr[0], (unsigned char *)&arr[2]) == true) {
		if(usb_Buffer[0] == 0x0E) {
			printf("Multiple digital outputs have been set\n");
		} else {
			printf("*** Error setting digital outputs, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (SetBlockDigitalOutputs)...\n"); }



	// Read digital input
	if(usb_ReadDigitalInput('B', 11) == true) {
		if(usb_Buffer[0] == CMD_READ_DIGITAL_INPUT) {
			printf("Input is %X\n", usb_Buffer[1]);
		} else {
			printf("*** Error reading digital input, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (ReadDigitalInput)...\n"); }


	// Read multiple digital inputs
	arr[0]= 0x0F;
	arr[1]= 0xFF;
	if(usb_ReadBlockDigitalInputs('B', (unsigned char *)&arr[0]) == true) {
		if(usb_Buffer[0] == CMD_READ_BLOCK_DIGITAL_INPUTS) {
			printf("Inputs are ");

			// lines below are applicable for A block
//			for(i=0; i<6; i++) {
//				if((usb_Buffer[2] & 0x20) == 0x20) {
//					printf("1");
//				} else {
//					printf("0"); }
//				usb_Buffer[2] <<= 1; }

			// lines below are applicable for B block
			for(i=0; i<4; i++) {
				if((usb_Buffer[1] & 0x08) == 0x08) {
					printf("1");
				} else {
					printf("0"); }
				usb_Buffer[1] <<= 1; }
			for(i=0; i<8; i++) {
				if((usb_Buffer[2] & 0x80) == 0x80) {
					printf("1");
				} else {
					printf("0"); }
				usb_Buffer[2] <<= 1; }
			//

			printf("\n");
		} else {
			printf("*** Error reading digital inputs, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (ReadBlockDigitalInputs)...\n"); }



	// Set PWM output
	if(usb_SetPWMOutput('A', 1, 75) == true) {
		if(usb_Buffer[0] == 0x0E) {
			printf("PWM output has been set\n");
		} else {
			printf("*** Error setting PWM output, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (SetPWMOutput)...\n"); }



	// Read analog input
	if(usb_ReadAnalogInput('B', 12) == true) {
		if(usb_Buffer[0] == CMD_READ_ANALOG_INPUT) {
			float Voltage;
			Voltage = (((float)usb_Buffer[2]*256) + (float)usb_Buffer[3]) / 1023;
			if(usb_Buffer[1] == '3') {
				printf("Power supply is 3.3V, ");
				Voltage *= 3.3f;
			} else {
				printf("Power supply is 5V, ");
				Voltage *= 5; }
			printf("input voltage = %1.3fV\n", Voltage);
		} else {
			printf("*** Error reading analog input, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (ReadAnalogInput)...\n"); }



	// Read multiple analog inputs - set mask to read all 12 analog inputs
	arr[0]= 0x0F;
	arr[1]= 0xFF;
	if(usb_ReadBlockAnalogInputs((unsigned char *)&arr[0]) == true) {
		if(usb_Buffer[0] == CMD_READ_BLOCK_ANALOG_INPUTS) {
			float Voltage;
			float Multiplier;
			if(usb_Buffer[1] == '3') {
				printf("Power supply is 3.3V.\n");
				Multiplier = 3.3f;
			} else {
				printf("Power supply is 5V.\n");
				Multiplier = 5; }

			printf("Input voltages are:\n");
			for(i=0; i<12; i++) {
				Voltage = ((((float)usb_Buffer[(i*2)+2] * 256) + (float)usb_Buffer[(i*2)+3]) / 1023) * Multiplier;
				printf("  input %d = %1.3fV\n", i+1, Voltage); }
			printf("\n");
		} else {
			printf("*** Error reading analog inputs, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (ReadBlockAnalogInputs)...\n"); }



	// Set SPI bus pins
	if(usb_SetSPIPins('A',4, 'A',3, 'A',2, 'A',1) == true) {
		if(usb_Buffer[0] == 0x0E) {
			printf("SPI bus pins have been set\n");
		} else {
			printf("*** Error setting SPI bus pins, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (SetSPIPins)...\n"); }



	// Write to SPI (RAM 23K256 - write 0xA5 to location 0x0001)
	arr[0] = 0x02; arr[1] = 0x00; arr[2] = 0x01; arr[3] = 0xA5;
	if(usb_WriteSPIBus(4, (unsigned char *)&arr[0]) == true) {
		printf("Data has been pushed to SPI bus\n");
	} else {
		printf("*** Error (WriteSPIBus)...\n"); }



	// Read SPI (RAM 23K256 - read one byte from location 0x0001)
	arr[0] = 0x03; arr[1] = 0x00; arr[2] = 0x01;
	if(usb_ReadSPIBus(3, 1, (unsigned char *)&arr[0]) == true) {
		if(usb_Buffer[0] == CMD_READ_SPI_BUS) {
			printf("Character read: %02X\n", usb_Buffer[2]);
		} else {
			printf("*** Error writing/reading SPI bus, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (ReadSPIBus)...\n"); }




	// Set I2C bus pins
	if(usb_SetI2CPins('B',5, 'B',6) == true) {
		if(usb_Buffer[0] == 0x0E) {
			printf("I2C bus pins have been set\n");
		} else {
			printf("*** Error setting I2C bus pins, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (SetI2CPins)...\n"); }



	// Write to I2C device (EEPROM 24LC512 - write bytes 0x12 0x34 0x56 0x78 starting at location 0x0001)
	arr[0] = 0xA2; arr[1] = 0x00; arr[2] = 0x01; arr[3] = 0x12; arr[4] = 0x34; arr[5] = 0x56; arr[6] = 0x78;
	if(usb_WriteI2CBus(7, (unsigned char *)&arr[0]) == true) {
		if(usb_Buffer[0] == 0x0E) {
			printf("Data was accepted by I2C device\n");
		} else {
			printf("*** Error writing to I2C device, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (WriteI2CBus)...\n"); }



	// Write to I2C device (EEPROM 24LC512 - set start location pointer to 0x0001)
	arr[0] = 0xA2; arr[1] = 0x00; arr[2] = 0x01;
	if(usb_WriteI2CBus(3, (unsigned char *)&arr[0]) == true) {
		if(usb_Buffer[0] == 0x0E) {
			printf("Data was accepted by I2C device\n");
		} else {
			printf("*** Error writing to I2C device, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (WriteI2CBus)...\n"); }

	// Read from I2C device (EEPROM 24LC512 - read 4 bytes, starting at location 0x0001, as set above)
	arr[0] = 0xA3;
	if(usb_ReadI2CBus(1, 4, (unsigned char *)&arr[0]) == true) {
		if(usb_Buffer[0] == CMD_READ_I2C_BUS) {
			printf("Characters read: %02X %02X %02X %02X\n", usb_Buffer[2], usb_Buffer[3], usb_Buffer[4], usb_Buffer[5]);
		} else {
			printf("*** Error writing/reading I2C bus, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (ReadI2CBus)...\n"); }




	// Set 1-Wire bus pins
	if(usb_Set1WirePin('B',7) == true) {
		if(usb_Buffer[0] == 0x0E) {
			printf("1-Wire bus pin has been set\n");
		} else {
			printf("*** Error setting 1-Wire bus pin, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (Set1WirePin)...\n"); }



	// Write to 1-Wire device (DS18S20 - initiate temperature conversion)
	arr[0] = 0xCC; arr[1] = 0x44;
	if(usb_Write1WireBus(2, (unsigned char *)&arr[0]) == true) {
		if(usb_Buffer[0] == 0x0E) {
			printf("Data was accepted by 1-Wire device\n");
		} else {
			printf("*** Error writing to 1-Wire device, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (Write1WireBus)...\n"); }



	// Read 1-Wire device (DS18S20 - read scratchpad)
	arr[0] = 0xCC; arr[1] = 0xBE;
	if(usb_Read1WireBus(2, 9, (unsigned char *)&arr[0]) == true) {
		if(usb_Buffer[0] == CMD_READ_1WIRE_BUS) {
			if(usb_Buffer[3] == 0) {	// positive temperature
				printf("Temperature: %2.1f deg C\n", (float)usb_Buffer[2] / 2);
			} else {		// negative temperature
				printf("Temperature: -%2.1f deg C\n", (256.0f - (float)usb_Buffer[2]) / 2); }
		} else {
			printf("*** Error writing/reading 1-Wire bus, Error code: %02X\n", usb_Buffer[0]); }
	} else {
		printf("*** Error (Read1WireBus)...\n"); }



	// Reset 1-Wire bus (the DQ bus pin is defined above with usb_Set1WirePin function)
	if(usb_Reset1WireBus() == true) {
		if(usb_Buffer[0] == CMD_RESET_1WIRE_BUS) {
			if(usb_Buffer[1] == 1) {
				printf("1-Wire bus reset done, at least one device responded\n");
			} else {
				printf("1-Wire bus reset done, no device responded\n"); }
		} else {
			printf("*** Error (Reset1WireBus)... Check that your silicon version is 0.5 or higher !\n"); }
	} else {
		printf("*** Error (Reset1WireBus)...\n"); }



	// Write bit '1' to 1-Wire bus
	if(usb_WriteBit1WireBus(false) == true) {
		if(usb_Buffer[0] == 0x0E) {
			printf("Bit timeslot created on the 1-Wire bus\n");
		} else {
			printf("*** Error (WriteBit1WireBus)... Check that your silicon version is 0.5 or higher !\n"); }
	} else {
		printf("*** Error (WriteBit1WireBus)...\n"); }



	// Read bit from 1-Wire bus
	if(usb_ReadBit1WireBus() == true) {
		if(usb_Buffer[0] == CMD_READ_BIT_1WIRE_BUS) {
			if(usb_Buffer[1] == 0) {
				printf("1-Wire bus bit state is 0 (low)\n");
			} else {
				printf("1-Wire bus bit state is 1 (high)\n"); }
		} else {
			printf("*** Error (ReadBit1WireBus)... Check that your silicon version is 0.5 or higher !\n"); }
	} else {
		printf("*** Error (ReadBit1WireBus)...\n"); }



	usb_CloseComm();
	return EXIT_SUCCESS;
}
