
/*
* FLYFISH TECHNOLOGIES Ltd.
* http://www.flyfish-tech.com
*
* FF3x library file
*   Updated: Aug 3, 2015
*
*
* This code is in the Public Domain. Use it at your own risk.
*/


#include "FF3x-lib.h"


char usb_DevicePath[32];
int usb_Handler;
unsigned char usb_Buffer[64];



bool usb_SendData(int pLen);


/////////////////////////

bool usb_GetChipInfo() {
	usb_Buffer[1] = CMD_GET_CHIP_INFO;
	return usb_SendData(1);
}

bool usb_SetAddress(unsigned char Addr) {
	usb_Buffer[1] = CMD_SET_ADDRESS;
	usb_Buffer[2] = Addr;
	return usb_SendData(2);
}

bool usb_GetAddress() {
	usb_Buffer[1] = CMD_GET_ADDRESS;
	return usb_SendData(1);
}

bool usb_SetVendor(unsigned char *Vendor, int pLen) {
	int i;
	usb_Buffer[1] = CMD_SET_VENDOR;
	if(pLen > 32)
		pLen = 32;
	for(i=0; i<pLen; i++)
		usb_Buffer[i+2] = Vendor[i];
	i++;
	while(i < 34) {
		i++;
		usb_Buffer[i] = 0; }
	return usb_SendData(34);
}

bool usb_GetVendor() {
	usb_Buffer[1] = CMD_GET_VENDOR;
	return usb_SendData(1);
}

bool usb_SetProduct(unsigned char *Product, int pLen) {
	int i;
	usb_Buffer[1] = CMD_SET_PRODUCT;
	if(pLen > 32)
		pLen = 32;
	for(i=0; i<pLen; i++)
		usb_Buffer[i+2] = Product[i];
	i++;
	while(i < 34) {
		i++;
		usb_Buffer[i] = 0; }
	return usb_SendData(34);
}

bool usb_GetProduct() {
	usb_Buffer[1] = CMD_GET_PRODUCT;
	return usb_SendData(1);
}

bool usb_SetSerialNumber(unsigned char *Serial, int pLen) {
	int i;
	usb_Buffer[1] = CMD_SET_SERIAL_NUMBER;
	if(pLen > 32)
		pLen = 32;
	for(i=0; i<pLen; i++)
		usb_Buffer[i+2] = Serial[i];
	i++;
	while(i < 34) {
		i++;
		usb_Buffer[i] = 0; }
	return usb_SendData(34);
}

bool usb_GetSerialNumber() {
	usb_Buffer[1] = CMD_GET_SERIAL_NUMBER;
	return usb_SendData(1);
}

bool usb_SetDigitalOutput(unsigned char PinBlock, unsigned char PinNumber, unsigned char State) {
	usb_Buffer[1] = CMD_SET_DIGITAL_OUTPUT;
	usb_Buffer[2] = PinBlock;
	usb_Buffer[3] = PinNumber;
	usb_Buffer[4] = State;
	return usb_SendData(4);
}

bool usb_SetBlockDigitalOutputs(unsigned char PinsBlock, unsigned char *PinsMask, unsigned char *States) {
	usb_Buffer[1] = CMD_SET_BLOCK_DIGITAL_OUTPUTS;
	usb_Buffer[2] = PinsBlock;
	usb_Buffer[3] = PinsMask[0];
	usb_Buffer[4] = PinsMask[1];
	usb_Buffer[5] = States[0];
	usb_Buffer[6] = States[1];
	return usb_SendData(6);
}

bool usb_ReadDigitalInput(unsigned char PinBlock, unsigned char PinNumber) {
	usb_Buffer[1] = CMD_READ_DIGITAL_INPUT;
	usb_Buffer[2] = PinBlock;
	usb_Buffer[3] = PinNumber;
	return usb_SendData(3);
}

bool usb_ReadBlockDigitalInputs(unsigned char PinsBlock, unsigned char *PinsMask) {
	usb_Buffer[1] = CMD_READ_BLOCK_DIGITAL_INPUTS;
	usb_Buffer[2] = PinsBlock;
	usb_Buffer[3] = PinsMask[0];
	usb_Buffer[4] = PinsMask[1];
	return usb_SendData(4);
}

bool usb_SetPWMOutput(unsigned char PinBlock, unsigned char PinNumber, unsigned char Ratio) {
	usb_Buffer[1] = CMD_SET_PWM_OUTPUT;
	usb_Buffer[2] = PinBlock;
	usb_Buffer[3] = PinNumber;
	usb_Buffer[4] = Ratio;
	return usb_SendData(4);
}

bool usb_ReadAnalogInput(unsigned char PinBlock, unsigned char PinNumber) {
	usb_Buffer[1] = CMD_READ_ANALOG_INPUT;
	usb_Buffer[2] = PinBlock;
	usb_Buffer[3] = PinNumber;
	return usb_SendData(3);
}

bool usb_ReadBlockAnalogInputs(unsigned char *PinsMask) {
	usb_Buffer[1] = CMD_READ_BLOCK_ANALOG_INPUTS;
	usb_Buffer[2] = PinsMask[0];
	usb_Buffer[3] = PinsMask[1];
	return usb_SendData(3);
}

bool usb_SetSPIPins(unsigned char CSPinBlock, unsigned char CSPinNumber, unsigned char SCKPinBlock, unsigned char SCKPinNumber, unsigned char MOSIPinBlock, unsigned char MOSIPinNumber, unsigned char MISOPinBlock, unsigned char MISOPinNumber) {
	usb_Buffer[1] = CMD_CONFIG_SPI_BUS;
	usb_Buffer[2] = CSPinBlock;
	usb_Buffer[3] = CSPinNumber;
	usb_Buffer[4] = SCKPinBlock;
	usb_Buffer[5] = SCKPinNumber;
	usb_Buffer[6] = MOSIPinBlock;
	usb_Buffer[7] = MOSIPinNumber;
	usb_Buffer[8] = MISOPinBlock;
	usb_Buffer[9] = MISOPinNumber;
	usb_Buffer[10] = 0;
	return usb_SendData(10);
}

bool usb_WriteSPIBus(int DataLen, unsigned char *Data) {
	int i;
	if(DataLen > 60) {
		return false; }
	usb_Buffer[1] = CMD_WRITE_SPI_BUS;
	usb_Buffer[2] = DataLen;
	for(i=0; i<DataLen; i++)
		usb_Buffer[i+3] = Data[i];
	return usb_SendData(DataLen+2);
}

bool usb_ReadSPIBus(int WRDataLen, int RDDataLen, unsigned char *WRData) {
	int i;
	if((WRDataLen > 60) || (RDDataLen > 60)) {
		return false; }
	usb_Buffer[1] = CMD_READ_SPI_BUS;
	usb_Buffer[2] = WRDataLen;
	usb_Buffer[3] = RDDataLen;
	for(i=0; i<WRDataLen; i++)
		usb_Buffer[i+4] = WRData[i];
	return usb_SendData(WRDataLen+3);
}

bool usb_SetI2CPins(unsigned char SCLPinBlock, unsigned char SCLPinNumber, unsigned char SDAPinBlock, unsigned char SDAPinNumber) {
	usb_Buffer[1] = CMD_CONFIG_I2C_BUS;
	usb_Buffer[2] = SCLPinBlock;
	usb_Buffer[3] = SCLPinNumber;
	usb_Buffer[4] = SDAPinBlock;
	usb_Buffer[5] = SDAPinNumber;
	return usb_SendData(5);
}

bool usb_WriteI2CBus(int DataLen, unsigned char *Data) {
	int i;
	if(DataLen > 60) {
		return false; }
	usb_Buffer[1] = CMD_WRITE_I2C_BUS;
	usb_Buffer[2] = DataLen;
	for(i=0; i<DataLen; i++)
		usb_Buffer[i+3] = Data[i];
	return usb_SendData(DataLen+2);
}

bool usb_ReadI2CBus(int WRDataLen, int RDDataLen, unsigned char *WRData) {
	int i;
	if((WRDataLen > 60) || (RDDataLen > 60)) {
		return false; }
	usb_Buffer[1] = CMD_READ_I2C_BUS;
	usb_Buffer[2] = WRDataLen;
	usb_Buffer[3] = RDDataLen;
	for(i=0; i<WRDataLen; i++)
		usb_Buffer[i+4] = WRData[i];
	return usb_SendData(WRDataLen+3);
}

bool usb_Set1WirePin(unsigned char DQPinBlock, unsigned char DQPinNumber) {
	usb_Buffer[1] = CMD_CONFIG_1WIRE_BUS;
	usb_Buffer[2] = DQPinBlock;
	usb_Buffer[3] = DQPinNumber;
	return usb_SendData(3);
}

bool usb_Reset1WireBus() {
	usb_Buffer[1] = CMD_RESET_1WIRE_BUS;
	return usb_SendData(1);
}

bool usb_Write1WireBus(int DataLen, unsigned char *Data) {
	int i;
	if(DataLen > 60) {
		return false; }
	usb_Buffer[1] = CMD_WRITE_1WIRE_BUS;
	usb_Buffer[2] = DataLen;
	for(i=0; i<DataLen; i++)
		usb_Buffer[i+3] = Data[i];
	return usb_SendData(DataLen+2);
}

bool usb_WriteBit1WireBus(bool BitValue) {
	usb_Buffer[1] = CMD_WRITE_BIT_1WIRE_BUS;
	usb_Buffer[2] = BitValue;
	return usb_SendData(2);
}

bool usb_Read1WireBus(int WRDataLen, int RDDataLen, unsigned char *WRData) {
	int i;
	if((WRDataLen > 60) || (RDDataLen > 60)) {
		return false; }
	usb_Buffer[1] = CMD_READ_1WIRE_BUS;
	usb_Buffer[2] = WRDataLen;
	usb_Buffer[3] = RDDataLen;
	for(i=0; i<WRDataLen; i++)
		usb_Buffer[i+4] = WRData[i];
	return usb_SendData(WRDataLen+3);
}

bool usb_ReadBit1WireBus() {
	usb_Buffer[1] = CMD_READ_BIT_1WIRE_BUS;
	return usb_SendData(1);
}


/////////////////////////

bool usb_SendData(int pLen) {
	int Count;
	usb_Buffer[0] = 0x00;
	Count = write(usb_Handler, usb_Buffer, pLen+1);
	if (Count < 0) {
		return false; }
	Count = read(usb_Handler, usb_Buffer, 64);
	if (Count < 0) {
		return false; }
	return true;
}

bool usb_OpenComm() {
	usb_Handler = open(usb_DevicePath, O_RDWR);
	if (usb_Handler < 0) {
		fprintf(stderr, "Unable to open USB device.\n");
		return false; }
	return true;
}

void usb_CloseComm() {
	close(usb_Handler);
}

bool usb_GetPath(int pCount) {
	struct udev *udev;
	struct udev_enumerate *enumerate;
	struct udev_list_entry *devices, *dev_list_entry;
	struct udev_device *dev;
	int Count = 0;

	usb_DevicePath[0] = 0x00;
	udev = udev_new();
	if (!udev) {
		return false; }
	enumerate = udev_enumerate_new(udev);
	udev_enumerate_add_match_subsystem(enumerate, "hidraw");
	udev_enumerate_scan_devices(enumerate);
	devices = udev_enumerate_get_list_entry(enumerate);
	udev_list_entry_foreach(dev_list_entry, devices) {
		const char *path;
		path = udev_list_entry_get_name(dev_list_entry);
		dev = udev_device_new_from_syspath(udev, path);
		strcpy(usb_DevicePath, udev_device_get_devnode(dev));
		dev = udev_device_get_parent_with_subsystem_devtype(dev, "usb", "usb_device");
		if (!dev) {
			return false; }
		if (udev_device_get_sysattr_value(dev, "product") != NULL) {
			if (strcmp(udev_device_get_sysattr_value(dev, "product"), "FF32") == 0) {
				Count++;
				if(Count == pCount) {
					udev_device_unref(dev);
					udev_enumerate_unref(enumerate);
					udev_unref(udev);
					return true; }	}	}
		udev_device_unref(dev); }
	udev_enumerate_unref(enumerate);
	udev_unref(udev);
	return false;
}
